@@ -21,12 +21,16 @@ __all__ = [
     "InterleavedPairedEndWriter",
     "TwoFilePairedEndReader",
     "TwoFilePairedEndWriter",
+    "MultipleFileReader",
+    "MultipleFastaWriter",
+    "MultipleFastqWriter",
     "read_chunks",
     "read_paired_chunks",
     "records_are_mates",
     "__version__",
 ]
 
+import functools
 from os import PathLike
 from typing import Optional, Union, BinaryIO
 
@@ -47,6 +51,13 @@ from .pairedend import (
     InterleavedPairedEndReader,
     InterleavedPairedEndWriter,
 )
+from .multipleend import (
+    MultipleFastaWriter,
+    MultipleFastqWriter,
+    MultipleFileReader,
+    MultipleFileWriter,
+    _open_multiple,
+)
 from .exceptions import (
     UnknownFileFormat,
     FileFormatError,
@@ -63,40 +74,50 @@ from .chunks import read_chunks, read_paired_chunks
 from ._version import version as __version__
 
 
-# Backwards compatibility aliases
+# Backwards compatibility alias
 Sequence = SequenceRecord
 
 
 def open(
-    file1: Union[str, PathLike, BinaryIO],
-    *,
+    *files: Union[str, PathLike, BinaryIO],
+    file1: Optional[Union[str, PathLike, BinaryIO]] = None,
     file2: Optional[Union[str, PathLike, BinaryIO]] = None,
     fileformat: Optional[str] = None,
     interleaved: bool = False,
     mode: str = "r",
     qualities: Optional[bool] = None,
-    opener=xopen
-) -> Union[SingleEndReader, PairedEndReader, SingleEndWriter, PairedEndWriter]:
+    opener=xopen,
+    compression_level: int = 1,
+    open_threads: int = 0,
+) -> Union[
+    SingleEndReader,
+    PairedEndReader,
+    SingleEndWriter,
+    PairedEndWriter,
+    MultipleFileReader,
+    MultipleFileWriter,
+]:
     """
-    Open one or two files in FASTA or FASTQ format for reading or writing.
+    Open one or more FASTQ or FASTA files for reading or writing.
 
     Parameters:
+      files:
+        one or more Path or open file-like objects. One for single-end
+        reads, two for paired-end reads etc. More than two files are also
+        supported. At least one file is required.
 
       file1:
-        Path or an open file-like object. For reading single-end reads, this is
-        the only required argument.
+        Deprecated keyword argument for the first file.
 
       file2:
-        Path or an open file-like object. When reading paired-end reads from
-        two files, set this to the second file.
+        Deprecated keyword argument for the second file.
 
       mode:
-        Either ``'r'`` or ``'rb'`` for reading, ``'w'`` for writing
-        or ``'a'`` for appending.
+        Set to ``'r'`` for reading, ``'w'`` for writing or ``'a'`` for appending.
 
       interleaved:
-        If True, then file1 contains interleaved paired-end data.
-        file2 must be None in this case.
+        If True, then there must be only one file argument that contains
+        interleaved paired-end data.
 
       fileformat:
         If *None*, the file format is autodetected from the file name
@@ -114,29 +135,68 @@ def open(
         - When False (no qualities available), an exception is raised when the
           auto-detected output format is FASTQ.
 
-      opener: A function that is used to open file1 and file2 if they are not
+      opener: A function that is used to open the files if they are not
         already open file-like objects. By default, ``xopen`` is used, which can
         also open compressed file formats.
 
-    Return:
-       A subclass of `SingleEndReader`, `PairedEndReader`, `SingleEndWriter` or
-       `PairedEndWriter`.
+      open_threads: By default, dnaio opens files in the main thread.
+        When threads is greater than 0, external processes are opened for
+        compressing and decompressing files. This decreases wall clock time
+        at the cost of a little extra overhead. This parameter does not work
+        when a custom opener is set.
+
+      compression_level: By default dnaio uses compression level 1 for writing
+        gzipped files as this is the fastest. A higher level can be set using
+        this parameter. This parameter does not work when a custom opener is
+        set.
     """
-    if mode not in ("r", "rb", "w", "a"):
-        raise ValueError("Mode must be 'r', 'rb', 'w' or 'a'")
-    elif interleaved and file2 is not None:
-        raise ValueError("When interleaved is True, file2 must be None")
-    elif interleaved or file2 is not None:
+    if files and (file1 is not None):
+        raise ValueError(
+            "The file1 keyword argument cannot be used together with files specified"
+            "as positional arguments"
+        )
+    elif len(files) > 1 and file2 is not None:
+        raise ValueError(
+            "The file2 argument cannot be used together with more than one "
+            "file specified as positional argument"
+        )
+    elif file1 is not None and file2 is not None and files:
+        raise ValueError(
+            "file1 and file2 arguments cannot be used together with files specified"
+            "as positional arguments"
+        )
+    elif file1 is not None and file2 is not None:
+        files = (file1, file2)
+    elif file2 is not None and len(files) == 1:
+        files = (files[0], file2)
+
+    if len(files) > 1 and interleaved:
+        raise ValueError("When interleaved is True, only one file must be specified.")
+    elif mode not in ("r", "w", "a"):
+        raise ValueError("Mode must be 'r', 'w' or 'a'")
+
+    if opener == xopen:
+        opener = functools.partial(
+            xopen, threads=open_threads, compresslevel=compression_level
+        )
+    if interleaved or len(files) == 2:
         return _open_paired(
-            file1,
-            file2=file2,
+            *files,
             opener=opener,
             fileformat=fileformat,
-            interleaved=interleaved,
             mode=mode,
             qualities=qualities,
         )
+    elif len(files) > 2:
+        return _open_multiple(
+            *files, fileformat=fileformat, mode=mode, qualities=qualities, opener=opener
+        )
+
     else:
         return _open_single(
-            file1, opener=opener, fileformat=fileformat, mode=mode, qualities=qualities
+            files[0],
+            opener=opener,
+            fileformat=fileformat,
+            mode=mode,
+            qualities=qualities,
         )
