@@ -1,4 +1,11 @@
-"""Chunked reading of FASTA and FASTQ files"""
+"""
+Chunked reading of FASTA and FASTQ files
+
+This can be used to very quickly split up the input file into similarly-sized chunks,
+without actually parsing the records. The chunks can then be distributed to worker threads
+or subprocess and be parsed and processed there.
+"""
+
 from io import RawIOBase
 from typing import Optional, Iterator, Tuple
 
@@ -13,10 +20,10 @@ def _fasta_head(buf: bytes, end: Optional[int] = None) -> int:
     Return an integer length such that buf[:length] contains the highest
     possible number of complete FASTA records.
     """
-    pos = buf.rfind(b'\n>', 0, end)
+    pos = buf.rfind(b"\n>", 0, end)
     if pos != -1:
         return pos + 1
-    if buf[0:1] == b'>' or buf[0:1] == b'#':
+    if buf[0:1] == b">" or buf[0:1] == b"#":
         return 0
     if len(buf) == 0:
         return 0
@@ -34,10 +41,10 @@ def _fastq_head(buf: bytes, end: Optional[int] = None) -> int:
     Two FASTQ records are required to ensure that read pairs in interleaved
     paired-end data are not split.
     """
-    linebreaks = buf.count(b'\n', 0, end)
+    linebreaks = buf.count(b"\n", 0, end)
     right = end
     for _ in range(linebreaks % 8 + 1):
-        right = buf.rfind(b'\n', 0, right)
+        right = buf.rfind(b"\n", 0, right)
     # Note that this works even if linebreaks == 0:
     # rfind() returns -1 and adding 1 gives index 0,
     # which is correct.
@@ -46,11 +53,23 @@ def _fastq_head(buf: bytes, end: Optional[int] = None) -> int:
 
 def read_chunks(f: RawIOBase, buffer_size: int = 4 * 1024**2) -> Iterator[memoryview]:
     """
-    Read a chunk of complete FASTA or FASTQ records from a file.
-    The size of a chunk is at most buffer_size.
-    f needs to be a file opened in binary mode.
+    Read chunks of complete FASTA or FASTQ records from a file.
+    If the format is detected to be FASTQ, all chunks except possibly the last contain
+    an even number of records such that interleaved paired-end reads remain in sync.
+    The yielded memoryview objects are only valid for one iteration because the internal
+    buffer is re-used in the next iteration.
 
-    The yielded memoryview objects become invalid on the next iteration.
+    Arguments:
+        f: File with FASTA or FASTQ reads; must have been opened in binary mode
+        buffer_size: Largest allowed chunk size
+
+    Yields:
+        memoryview representing the chunk. This becomes invalid on the next iteration.
+
+    Raises:
+         ValueError: A FASTQ record was encountered that is larger than *buffer_size*.
+         UnknownFileFormat: The file format could not be detected
+           (the first byte must be "@", ">" or "#")
     """
     # This buffer is re-used in each iteration.
     buf = bytearray(buffer_size)
@@ -62,9 +81,9 @@ def read_chunks(f: RawIOBase, buffer_size: int = 4 * 1024**2) -> Iterator[memory
         # Empty file
         return
     assert start == 1
-    if buf[0:1] == b'@':
+    if buf[0:1] == b"@":
         head = _fastq_head
-    elif buf[0:1] == b'#' or buf[0:1] == b'>':
+    elif buf[0:1] == b"#" or buf[0:1] == b">":
         head = _fasta_head
     else:
         raise UnknownFileFormat(
@@ -87,7 +106,7 @@ def read_chunks(f: RawIOBase, buffer_size: int = 4 * 1024**2) -> Iterator[memory
 
     while True:
         if start == len(buf):
-            raise OverflowError('FASTA/FASTQ record does not fit into buffer')
+            raise OverflowError("FASTA/FASTQ record does not fit into buffer")
         bufend = f.readinto(memoryview(buf)[start:]) + start  # type: ignore
         if start == bufend:
             # End of file
@@ -109,6 +128,29 @@ def read_paired_chunks(
     f2: RawIOBase,
     buffer_size: int = 4 * 1024**2,
 ) -> Iterator[Tuple[memoryview, memoryview]]:
+    """
+    Read chunks of paired-end FASTQ reads from two files.
+    A pair of chunks (memoryview objects) is yielded on each iteration,
+    and both chunks are guaranteed to have the same number of sequences.
+    That is, the paired-end reads will stay in sync.
+
+    The memoryviews are only valid for one iteration because the internal
+    buffer is re-used in the next iteration.
+
+    This is similar to `read_chunks`, but for paired-end data.
+    Unlike `read_chunks`, this only works for FASTQ input.
+
+    Args:
+        f: File with R1 reads; must have been opened in binary mode
+        f2: File with R2 reads; must have been opened in binary mode
+        buffer_size: Largest allowed chunk size
+
+    Yields:
+        Pairs of memoryview objects.
+
+    Raises:
+         ValueError: A FASTQ record was encountered that is larger than *buffer_size*.
+    """
     if buffer_size < 1:
         raise ValueError("Buffer size too small")
 
@@ -118,9 +160,11 @@ def read_paired_chunks(
     # Read one byte to make sure we are processing FASTQ
     start1 = f.readinto(memoryview(buf1)[0:1])  # type: ignore
     start2 = f2.readinto(memoryview(buf2)[0:1])  # type: ignore
-    if (start1 == 1 and buf1[0:1] != b'@') or (start2 == 1 and buf2[0:1] != b'@'):
+    if (start1 == 1 and buf1[0:1] != b"@") or (start2 == 1 and buf2[0:1] != b"@"):
         raise FileFormatError(
-            "Paired-end data must be in FASTQ format when using multiple cores", line=None)
+            "Paired-end data must be in FASTQ format when using multiple cores",
+            line=None,
+        )
 
     while True:
         if start1 == len(buf1) or start2 == len(buf2):
