@@ -22,8 +22,10 @@ from dnaio import (
     FastqWriter,
     InterleavedPairedEndWriter,
     TwoFilePairedEndReader,
+    records_are_mates,
+    record_names_match,
+    SequenceRecord,
 )
-from dnaio import records_are_mates, record_names_match, SequenceRecord
 from dnaio.writers import FileWriter
 from dnaio.readers import BinaryFileReader
 
@@ -264,10 +266,10 @@ class TestFastqReader:
 
 
 class TestOpen:
-    def setup(self):
+    def setup_method(self):
         self._tmpdir = mkdtemp()
 
-    def teardown(self):
+    def teardown_method(self):
         shutil.rmtree(self._tmpdir)
 
     def test_sequence_reader(self):
@@ -339,7 +341,7 @@ class TestOpen:
         path = os.path.join(self._tmpdir, "tmp.fastq")
         with raises(ValueError):
             with dnaio.open(path, mode="w", qualities=False):
-                pass
+                pass  # pragma: no cover
 
 
 class TestInterleavedReader:
@@ -386,11 +388,11 @@ class TestInterleavedReader:
 
 
 class TestFastaWriter:
-    def setup(self):
+    def setup_method(self):
         self._tmpdir = mkdtemp()
         self.path = os.path.join(self._tmpdir, "tmp.fasta")
 
-    def teardown(self):
+    def teardown_method(self):
         shutil.rmtree(self._tmpdir)
 
     def test(self):
@@ -436,11 +438,11 @@ class TestFastaWriter:
 
 
 class TestFastqWriter:
-    def setup(self):
+    def setup_method(self):
         self._tmpdir = mkdtemp()
         self.path = os.path.join(self._tmpdir, "tmp.fastq")
 
-    def teardown(self):
+    def teardown_method(self):
         shutil.rmtree(self._tmpdir)
 
     def test(self):
@@ -608,7 +610,7 @@ def test_file_writer(tmp_path):
     assert path.exists()
     with raises(ValueError) as e:
         with fw:
-            pass
+            pass  # pragma: no coverage
     assert "operation on closed file" in e.value.args[0]
 
 
@@ -618,7 +620,7 @@ def test_binary_file_reader():
     bfr.close()
     with raises(ValueError) as e:
         with bfr:
-            pass
+            pass  # pragma: no coverage
     assert "operation on closed" in e.value.args[0]
 
 
